package im.zego.liveshow.effects;

import android.content.Context;

import java.util.ArrayList;
import java.util.List;

import im.zego.liveshow.R;
import im.zego.liveshow.model.AudioEffectInfo;
import im.zego.liveshow.utils.CopyAssetsFileUtil;
import im.zego.zegoexpress.ZegoAudioEffectPlayer;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.constants.ZegoReverbPreset;
import im.zego.zegoexpress.constants.ZegoVoiceChangerPreset;
import im.zego.zegoexpress.entity.ZegoAudioEffectPlayConfig;

/**
 * 音乐特效管理类
 */
public class SoundEffectsManager {

    private static volatile SoundEffectsManager singleton = null;
    private ZegoExpressEngine engine;
    private ZegoAudioEffectPlayer mAudioEffectPlayer;

    private int backgroundSoundPosition = -1; //背景音乐默认值
    private int voiceChangePosition = 0; //变声默认值
    private int reverbPresetPosition = 0; //混响

    private int musicVolum = 50; //音乐音量
    private int vocalVolume = 50; //人声音量

    private List<AudioEffectInfo> mAudioEffectInfo = new ArrayList<>();
    private ArrayList<String> mSongFilePaths = new ArrayList<>();

    public static SoundEffectsManager getInstance() {
        if (singleton == null) {
            synchronized (SoundEffectsManager.class) {
                if (singleton == null) {
                    singleton = new SoundEffectsManager();
                }
            }
        }
        return singleton;
    }

    public SoundEffectsManager() {
        engine = ZegoExpressEngine.getEngine();
        if(engine != null){
            mAudioEffectPlayer = engine.createAudioEffectPlayer();
        }
    }

    /**
     * 设置变声
     *
     * @param voicePreset
     */
    public void setVoiceChangerPreset(String voicePreset) {
        switch (voicePreset) {
            case "NONE": //无
                engine.setVoiceChangerPreset(ZegoVoiceChangerPreset.NONE);
                break;
            case "WOMEN_TO_CHILD": //萝莉
                engine.setVoiceChangerPreset(ZegoVoiceChangerPreset.WOMEN_TO_CHILD);
                break;
            case "FOREIGNER": //外国人
                engine.setVoiceChangerPreset(ZegoVoiceChangerPreset.FOREIGNER);
                break;
            case "ANDROID": //机器人
                engine.setVoiceChangerPreset(ZegoVoiceChangerPreset.ANDROID);
                break;
            case "ETHEREAL": //空灵
                engine.setVoiceChangerPreset(ZegoVoiceChangerPreset.ETHEREAL);
                break;
        }
    }

    /**
     * 设置混响
     *
     * @param reverbPreset
     */
    public void setReverbPreset(String reverbPreset) {
        switch (reverbPreset) {
            case "NONE": //无
                engine.setReverbPreset(ZegoReverbPreset.NONE);
                break;
            case "KTV": //ktv
                engine.setReverbPreset(ZegoReverbPreset.KTV);
                break;
            case "CONCERT_HALL": //音乐厅
                engine.setReverbPreset(ZegoReverbPreset.CONCERT_HALL);
                break;
            case "VOCAL_CONCERT": //演唱会
                engine.setReverbPreset(ZegoReverbPreset.VOCAL_CONCERT);
                break;
            case "ROCK":  //摇滚
                engine.setReverbPreset(ZegoReverbPreset.ROCK);
                break;
        }
    }

    /**
     * 指定的音效音量，取值范围为 [0, 200]，默认值为 “100”
     */
    public void setVolume(int volume) {
        mAudioEffectPlayer.setVolumeAll(volume * 2);
    }

    /**
     * 设置推流端采集音量 音量百分比，范围为 0 ~ 200，默认值为 100。
     *
     * @param volume
     */
    public void setCaptureVolume(int volume) {
        engine.setCaptureVolume(volume * 2);
    }

    /**
     * 开始播放音效
     *
     * @param audioEffectID
     * @param path
     */
    public void audioEffectPlayer(int audioEffectID, String path) {
        ZegoAudioEffectPlayConfig config = new ZegoAudioEffectPlayConfig();
        config.playCount = 0;
        config.isPublishOut = true;
        mAudioEffectPlayer.start(audioEffectID, path, config);
    }

    /**
     * 音效暂停播放
     *
     * @param audioEffectID
     */
    public void audioEffectStop(int audioEffectID) {
        mAudioEffectPlayer.stop(audioEffectID);
    }


    /**
     * copy mp3 数据
     */
    public void initLocalAudioEffectlist(Context mContext) {

        mAudioEffectInfo.clear();
        String[] musicList = mContext.getResources().getStringArray(R.array.background_music);
        for (int i = 0; i < musicList.length; ++i) {
            String musicName = musicList[i];
            String mFileAssetsPath = musicName + ".mp3";
            mAudioEffectInfo.add(new AudioEffectInfo().setAudioEffectId(i).setFileAssetsPath(mFileAssetsPath));
        }

        new Thread(new Runnable() {
            @Override
            public void run() {
                copyAssetsToLocalStorage(mContext);
            }
        }).start();
    }

    /**
     * 拷贝背景音到本地
     */
    private void copyAssetsToLocalStorage(Context mContext) {
        mSongFilePaths.clear();
        for (AudioEffectInfo audioEffectInfo : mAudioEffectInfo) {
            String path = CopyAssetsFileUtil.copyAssetsFile2Phone(mContext, audioEffectInfo.getFileAssetsPath());
            mSongFilePaths.add(path);
        }
    }


    public void clearSoundEffectsData() {

        if(mAudioEffectPlayer != null){
            setVolume(50);
            mAudioEffectPlayer.stopAll();
        }
        backgroundSoundPosition = -1; //背景音乐默认值
        voiceChangePosition = 0; //变声默认值
        reverbPresetPosition = 0; //混响
        musicVolum = 50; //音乐音量
        vocalVolume = 50; //人声音量
        if(engine != null){
            setCaptureVolume(50);
            engine.setReverbPreset(ZegoReverbPreset.NONE);
            engine.setVoiceChangerPreset(ZegoVoiceChangerPreset.NONE);
        }
    }

    public void destroyAudioEffect() {
        if(engine != null){
            engine.destroyAudioEffectPlayer(mAudioEffectPlayer);
        }
        singleton = null;
    }

    public int getBackgroundSoundPosition() {
        return backgroundSoundPosition;
    }

    public void setBackgroundSoundPosition(int backgroundSoundPosition) {
        this.backgroundSoundPosition = backgroundSoundPosition;
    }

    public int getVoiceChangePosition() {
        return voiceChangePosition;
    }

    public void setVoiceChangePosition(int voiceChangePosition) {
        this.voiceChangePosition = voiceChangePosition;
    }

    public int getReverbPresetPosition() {
        return reverbPresetPosition;
    }

    public void setReverbPresetPosition(int reverbPresetPosition) {
        this.reverbPresetPosition = reverbPresetPosition;
    }

    public int getMusicVolum() {
        return musicVolum;
    }

    public void setMusicVolum(int musicVolum) {
        this.musicVolum = musicVolum;
    }

    public int getVocalVolume() {
        return vocalVolume;
    }

    public void setVocalVolume(int vocalVolume) {
        this.vocalVolume = vocalVolume;
    }

    public ArrayList<String> getmSongFilePaths() {
        return mSongFilePaths;
    }

}
