package im.zego.common.util;

import android.os.Handler;
import android.os.Looper;

import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

public class ThreadHelper {

    public static final int CPU_COUNT = Runtime.getRuntime().availableProcessors();

    public static final Handler handler = new Handler(Looper.getMainLooper());

    public static final ConcurrentHashMap<Byte, ExecutorService> executorMap = new ConcurrentHashMap();

    /**
     * 线程类型
     */
    private static final byte TYPE_SINGLE = -1;
    private static final byte TYPE_CACHED = -2;
    private static final byte TYPE_IO = -3;


    /**
     * 当前是否在主线程中
     *
     * @return true:在主线程 false:不在
     */
    public static boolean isMainThread() {
        return Looper.myLooper() == Looper.getMainLooper();
    }

    /**
     * ui线程任务执行
     *
     * @param runnable
     */
    public static void runOnUiThread(Runnable runnable) {
        runOnUiThread(runnable, 0);
    }

    /**
     * ui线程任务执行
     *
     * @param runnable
     * @param delay    延时时间，毫秒
     */
    public static void runOnUiThread(Runnable runnable, long delay) {
        if (isMainThread() && delay <= 0) {
            runnable.run();
        } else {
            handler.postDelayed(runnable, delay);
        }
    }

    /**
     * 子线程中执行任务
     * 如果在子线程中，会在当前线程直接执行任务，会阻塞线程
     *
     * @param runnable
     */
    public static void runIO(Runnable runnable) {
        if (isMainThread()) {
            executeIO(runnable);
        } else {
            runnable.run();
        }
    }

    /**
     * 新线程执行
     * 如果在子线程中，也重新创建线程
     *
     * @param runnable
     */
    public static void executeIO(Runnable runnable) {
        getExecutorService(TYPE_IO).execute(runnable);
    }

    /**
     * 阻塞任务
     *
     * @param callable
     */
    public static <T> Future<T> submitIO(Callable<T> callable) {
        return getExecutorService(TYPE_IO).submit(callable);
    }

    /**
     * 阻塞任务
     *
     * @param runnable
     * @return
     */
    public static Future submitIO(Runnable runnable) {
        return getExecutorService(TYPE_IO).submit(runnable);
    }

    /**
     * 获取所属线程池
     *
     * @param type 线程池类型
     * @return
     */
    private static ExecutorService getExecutorService(byte type) {
        synchronized (executorMap) {
            ExecutorService service = executorMap.get(type);
            if (service == null) {
                service = new ThreadPoolExecutor(
                        2 * CPU_COUNT + 1,
                        Integer.MAX_VALUE,
                        30, TimeUnit.SECONDS,
                        new SynchronousQueue(),
                        new LocalThreadFactory("ThreadHelper#Thread")
                );
                executorMap.put(type, service);
            }
            return service;
        }
    }

    private static final class LocalThreadFactory implements ThreadFactory {
        private String threadName;
        private AtomicInteger statics = new AtomicInteger();

        public LocalThreadFactory(String threadName) {
            this.threadName = threadName;
        }

        @Override
        public Thread newThread(Runnable r) {

            return new Thread(r, threadName());
        }

        private String threadName() {
            statics.incrementAndGet();
            return new StringBuffer()
                    .append(threadName)
                    .append("(")
                    .append(statics.get())
                    .append(")")
                    .toString();
        }
    }

}
