package im.zego.gochat.chatroom;


import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.net.ssl.SNIHostName;

import im.zego.common.application.ZegoApplication;
import im.zego.gochat.ErrorcodeConstants;
import im.zego.gochat.MainActivity;
import im.zego.gochat.R;
import im.zego.gochat.model.NotifyInvitedInfo;
import im.zego.gochat.model.NotifyRefuseInviteInfo;
import im.zego.gochat.model.NotifyUserListUpdateInfo;
import im.zego.gochat.model.NotifyUserStateUpdateInfo;
import im.zego.gochat.model.RaiseHandUserInfoData;
import im.zego.gochat.model.RoomInfo;
import im.zego.gochat.model.UserInfo;
import im.zego.gochat.model.UserInfoData;
import im.zego.gochat.protocol.ICommonResponseCallback;
import im.zego.gochat.protocol.IGetRaiseHandUserListCallback;
import im.zego.gochat.protocol.IGetUserListCallback;
import im.zego.gochat.protocol.ILoginRoomCallback;
import im.zego.gochat.protocol.RoomAPI;
import im.zego.gochat.rtc.RTCSDKManager;
import im.zego.gochat.rtc.callbacks.IRTCCommonCallback;
import im.zego.gochat.rtc.callbacks.IRTCEventCallback;
import im.zego.gochat.utils.ToastUtil;

import static im.zego.gochat.ErrorcodeConstants.ERROR_USER_OFF_LINE;
import static im.zego.gochat.ErrorcodeConstants.GoChatErrorFailNetwork;

/**
 * 房间管理类
 */
public class ChatRoomManager {

    private static final String TAG = "ChatRoomManager";

    private static volatile ChatRoomManager singleton = null;
    private RoomInfo mRoomInfo;
    protected HashMap<Long, UserInfo> mUserModelMap = new HashMap<>();
    private UserInfo myUserInfo;
    private final List<UserInfo> raiseHandList = new ArrayList<>();

    private HashMap<Long, UserInfo> mRaiseMap = new HashMap<>();

    private int raiseHandNum = 0;
    private boolean hasMoreUser = true;
    private final static int COUNT_SIZE = 500;
    private int page = 1;
    private static final Handler handler = new Handler(Looper.getMainLooper());

    private ChatRoomManager() {
    }

    public static ChatRoomManager getInstance() {
        if (singleton == null) {
            synchronized (ChatRoomManager.class) {
                if (singleton == null) {
                    singleton = new ChatRoomManager();
                }
            }
        }
        return singleton;
    }

    /**
     * 房间信息
     *
     * @return
     */
    public RoomInfo getRoomInfo() {
        return mRoomInfo;
    }

    /**
     * 用户列表
     *
     * @return
     */
    public HashMap<Long, UserInfo> getUserModelMap() {
        return mUserModelMap;
    }

    /**
     * 获取自己的用户信息
     *
     * @return
     */
    public UserInfo getMyUserInfo() {
        return myUserInfo;
    }

    public List<UserInfo> getRaiseHandList() {
        return raiseHandList;
    }

    /**
     * 登录房间
     *
     * @param uid
     * @param roomId
     */
    public void loginRoom(final Context context, long uid, final String roomId) {
        RoomAPI.loginRoom(uid, roomId, new ILoginRoomCallback() {
            // TODO: 2/13/21
            @Override
            public void onLoginRoom(int retcode, RoomInfo roomInfo) {
                if (retcode == 0) {
                    setRoomInfo(roomInfo);
                    context.startActivity(new Intent(context, ChatRoomActivity.class));
                } else if (retcode == ErrorcodeConstants.ERROR_ROOM_FULL) {
                    ToastUtil.showWarnToast(R.string.room_list_join_fail);
                } else if (retcode == ErrorcodeConstants.ERROR_ALREADY_IN_ROOM) {
                    ToastUtil.showWarnToast(R.string.room_list_join_fail_common);
                } else if (retcode == ERROR_USER_OFF_LINE) {
                    ToastUtil.showWarnToast(R.string.room_list_join_fail_common);
                    MainActivity.restartLogin();
                } else if (retcode == GoChatErrorFailNetwork) {
                    ToastUtil.showWarnToast(R.string.fail_network);
                } else {
                    ToastUtil.showWarnToast(R.string.room_list_join_fail_common);
                }
            }
        });
    }

    public void setRoomInfo(RoomInfo roomInfo) {
        mRoomInfo = roomInfo;
        // 不是所有路径都会走 RoomAPI.loginRoom，但是都会走 setRoomInfo，所以RTC的相关设置在这里
        RTCSDKManager.getInstance().addFirstListener(mRTCEventCallback);
    }

    public void logoutRoom() {
        // 保底处理
        RTCSDKManager.getInstance().stopPublishAudio(null);
        RTCSDKManager.getInstance().logoutRoom();
        RTCSDKManager.getInstance().removeListener(mRTCEventCallback);

        if (myUserInfo != null) {
            RoomAPI.quitRoom(myUserInfo.getUserID(), null);
        }
        mUserModelMap.clear();
        mRoomInfo = null;
        myUserInfo = null;
        raiseHandList.clear();
        mRaiseMap.clear();
        raiseHandNum = 0;
    }

    /**
     * 拉取用户列表,最大5000人，分10次拉取，间隔1s
     *
     * @param uid
     * @param callback
     */
    public void getUserList(final long uid, final IExecuteCallback callback) {
        hasMoreUser = true;
        page = 1;
        // 先清除之前的列表
        mUserModelMap.clear();
        RoomAPI.getUserList(uid, page, COUNT_SIZE, new IGetUserListCallback() {
            @Override
            public void onGetUserList(int retcode, UserInfoData userInfoData) {
                if (retcode == 0 && userInfoData != null) {
                    List<UserInfo> attendeeList = userInfoData.getAttendeeList();
                    for (UserInfo userInfo : attendeeList) {
                        mUserModelMap.put(userInfo.getUserID(), userInfo);
                    }
                    if (attendeeList != null && attendeeList.size() == COUNT_SIZE) {
                        page++;
                        getLoopUserList(uid);
                    } else {
                        hasMoreUser = false;
                        myUserInfo = mUserModelMap.get(uid);
                        callback.onSuccess();
                    }
                } else {
                    hasMoreUser = false;
                    callback.onFailed(retcode);
                }
            }

        });
    }

    /**
     * 循环拉取列表
     *
     * @param uid
     */

    private void getLoopUserList(long uid) {
        if (page > 10 || mRoomInfo == null)
            return;
        if (hasMoreUser) {
            handler.postDelayed(() -> RoomAPI.getUserList(uid, page, COUNT_SIZE, new IGetUserListCallback() {
                @Override
                public void onGetUserList(int retcode, UserInfoData userInfoData) {
                    if (retcode == 0 && userInfoData != null) {
                        List<UserInfo> attendeeList = userInfoData.getAttendeeList();
                        for (UserInfo userInfo : attendeeList) {
                            mUserModelMap.put(userInfo.getUserID(), userInfo);
                        }
                        if (attendeeList != null && attendeeList.size() == COUNT_SIZE) {
                            page++;
                            getLoopUserList(uid);
                        } else {
                            hasMoreUser = false;
                            myUserInfo = mUserModelMap.get(uid);
                        }
                    } else {
                        hasMoreUser = false;
                    }
                }
            }), 1000);
        }
    }


    /**
     * 拉取举手列表
     *
     * @param uid
     * @param page
     * @param size
     * @param callback
     */
    public void getRaiseHandList(final long uid, int page, int size, final IExecuteCallback callback) {
        RoomAPI.getRaiseHandList(uid, page, size, new IGetRaiseHandUserListCallback() {
            @Override
            public void onGetRaiseHandUserList(int retcode, RaiseHandUserInfoData raiseHandUserInfoData) {
                if (retcode == 0) {
                    // 先清除之前的列表
                    raiseHandList.clear();
                    if (raiseHandUserInfoData != null) {
                        raiseHandList.addAll(raiseHandUserInfoData.getRaiseHandList());
                    }
                    callback.onSuccess();
                } else {
                    callback.onFailed(retcode);
                }
            }
        });
    }


    /**
     * 关闭房间
     *
     * @param uid
     * @param callback
     */
    public void closeRoom(final long uid, final IExecuteCallback callback) {
        RoomAPI.closeRoom(uid, new ICommonResponseCallback() {
            @Override
            public void onResponse(int retcode) {
                if (retcode == 0) {
                    //结束房间成功，清楚数据
                    callback.onSuccess();
                } else {
                    callback.onFailed(retcode);
                }
            }
        });
    }

    /**
     * 举手
     *
     * @param uid
     * @param callback
     */
    public void raiseHand(final long uid, final boolean isRaiseHand, final IExecuteCallback callback) {
        RoomAPI.raiseHand(uid, isRaiseHand, new ICommonResponseCallback() {
            @Override
            public void onResponse(int retcode) {
                if (retcode == 0) {
                    callback.onSuccess();
                    if (isRaiseHand) {
                        // 若是放下手，不弹toast
                        ToastUtil.showNormalToast(R.string.raise_hand_tip_success);
                    }
                } else {
                    callback.onFailed(retcode);
                }
            }
        });
    }

    /**
     * 邀请上台
     *
     * @param uid
     * @param callback
     */
    public void inviteOnstage(final long uid, final long targetUid, final IExecuteCallback callback) {
        RoomAPI.inviteOnstage(uid, targetUid, new ICommonResponseCallback() {
            @Override
            public void onResponse(int retcode) {
                if (retcode == 0) {
                    callback.onSuccess();
                } else {
                    callback.onFailed(retcode);
                }
            }
        });
    }


    /**
     * 回应邀请上台
     *
     * @param uid
     * @param token
     * @param response //1:  拒绝  2：同意
     * @param callback
     */
    public void responseInviteOnstage(final long uid, final String token, int response, final IExecuteCallback callback) {
        RoomAPI.responseinviteOnstage(uid, token, response, new ICommonResponseCallback() {
            @Override
            public void onResponse(int retcode) {
                if (retcode == 0) {
                    callback.onSuccess();
                } else {
                    callback.onFailed(retcode);
                }
            }
        });
    }

    /**
     * 同意邀请上台
     *
     * @param uid
     * @param token
     * @param callback
     */
    public void responseAgreeInviteOnstage(final long uid, final String token, final IExecuteCallback callback) {
        this.responseInviteOnstage(uid, token, 2, callback);
    }

    /**
     * 拒绝邀请上台
     *
     * @param uid
     * @param token
     * @param callback
     */
    public void responseRejectInviteOnstage(final long uid, final String token, final IExecuteCallback callback) {
        this.responseInviteOnstage(uid, token, 1, callback);
    }

    /**
     * 设置用户状态
     *
     * @param uid
     * @param callback
     */
    public void setUserInfo(final long uid, final long targetUid, boolean isMicOpen, int onstage, int type, final IExecuteCallback callback) {
        RoomAPI.setUserInfo(uid, targetUid, isMicOpen, onstage, type, retcode -> {
            if (retcode == 0) {
                callback.onSuccess();
            } else {
                callback.onFailed(retcode);
            }
        });
    }


    private IRaiseHandNumChangeNotify mIRaiseHandNumChangeNotify;

    public void setOnRaiseHandNumChangeNotify(IRaiseHandNumChangeNotify iRaiseHandNumChangeNotify) {
        mIRaiseHandNumChangeNotify = iRaiseHandNumChangeNotify;
    }


    public synchronized void addRaiseHandUser(UserInfo userInfo) {
        if (mRaiseMap.get(userInfo.getUserID()) == null) {
            raiseHandNum++;
            if (mIRaiseHandNumChangeNotify != null) {
                mIRaiseHandNumChangeNotify.onRaiseHandNumChange(raiseHandNum);
            }
        }
        mRaiseMap.put(userInfo.getUserID(), userInfo);

    }

    public synchronized void removeRaiseHandUser(long userId) {
        if (mRaiseMap.remove(userId) != null) {
            raiseHandNum--;
            if (mIRaiseHandNumChangeNotify != null) {
                mIRaiseHandNumChangeNotify.onRaiseHandNumChange(raiseHandNum);
            }
        }
    }

    public int getRaiseHandUserNum() {
        return raiseHandNum;
    }


    private IRTCEventCallback mRTCEventCallback = new IRTCEventCallback() {
        @Override
        public void onUserListUpdateNotify(@NotNull NotifyUserListUpdateInfo notifyInfo) {
            for (NotifyUserListUpdateInfo.UserUpdateInfo updateInfo : notifyInfo.users) {
                if (updateInfo.delta == -1) {
                    removeRaiseHandUser(updateInfo.uid);
                }
            }

        }

        @Override
        public void onUserStateUpdateNotify(@NotNull NotifyUserStateUpdateInfo notifyInfo) {
            if (notifyInfo.type == 1) { // 举手变更通知
                for (NotifyUserStateUpdateInfo.UserStateInfo userStateInfo : notifyInfo.users) {
                    if (userStateInfo.getUserID() == myUserInfo.getUserID()) {
                        // 更新自己的userInfo
                        myUserInfo = userStateInfo.toUserInfo();
                    }

                    if (userStateInfo.isRaisedHand()) {
                        addRaiseHandUser(userStateInfo.toUserInfo());
                    } else {
                        removeRaiseHandUser(userStateInfo.getUserID());
                    }
                }
            } else if (notifyInfo.type == 2) { // role变更通知
                for (NotifyUserStateUpdateInfo.UserStateInfo userStateInfo : notifyInfo.users) {
                    Log.d(TAG, String.format("user:%d nickName:%s newRole:%d", userStateInfo.getUserID(), userStateInfo.getNickName(), userStateInfo.getRole()));

                    if (userStateInfo.getUserID() == myUserInfo.getUserID()) {
                        if (userStateInfo.isHost() || userStateInfo.isStageGuest()) {
                            // 自己上台的时候开始推流
                            RTCSDKManager.getInstance().startPublishAudio(new IRTCCommonCallback() {
                                @Override
                                public void onRTCCallback(int errorCode) {

                                }
                            });
                        } else {
                            // 自己下台的时候关闭推流
                            RTCSDKManager.getInstance().stopPublishAudio(null);
                        }

                        // 更新自己的userInfo
                        myUserInfo = userStateInfo.toUserInfo();
                        myUserInfo.setRaiseHandStatus(false);

                        if (userStateInfo.isStageGuest()) {
                            removeRaiseHandUser(userStateInfo.getUserID());
                        }

                    } else {
                        if (userStateInfo.isHost()) {
                            // 别人上台，这里不拉取别人的声音流，完全依赖底层自动完成
                        } else if (userStateInfo.isStageGuest()) {
                            removeRaiseHandUser(userStateInfo.getUserID());
                            // 别人上台，这里不拉取别人的声音流，完全依赖底层自动完成
                        } else {
                            // 别人下台，停止拉流（可能不需要，保底处理）
                            RTCSDKManager.getInstance().stopPlayingAudio(userStateInfo.getUserID());
                        }


                    }
                }
            } else if (notifyInfo.type == 3) {  // mic变更通知
                Log.d(TAG, "mic变更通知 count:%d" + notifyInfo.users.size());
                for (NotifyUserStateUpdateInfo.UserStateInfo userStateInfo : notifyInfo.users) {
                    UserInfo notifyUserInfo = userStateInfo.toUserInfo();
                    Log.d(TAG, String.format("user:%d nickName:%s new micStatus:%s", notifyUserInfo.getUserID(), notifyUserInfo.getNickName(), notifyUserInfo.isMicOpen()));
                    if (notifyUserInfo.getUserID() == myUserInfo.getUserID()) {
                        if (notifyUserInfo.isMicOpen()) {
                            RTCSDKManager.getInstance().muteMicrophone(false);
                            RTCSDKManager.getInstance().startPublishAudio(new IRTCCommonCallback() {
                                @Override
                                public void onRTCCallback(int errorCode) {

                                }
                            });
                        } else {
                            RTCSDKManager.getInstance().muteMicrophone(true);
                            RTCSDKManager.getInstance().stopPublishAudio(null);
                        }

                        // 更新自己的userInfo
                        myUserInfo = notifyUserInfo;
                    } else {
                        // 这里不拉取别人的声音流，完全依赖底层自动完成
                        if (!notifyUserInfo.isMicOpen()) {
                            RTCSDKManager.getInstance().stopPlayingAudio(notifyUserInfo.getUserID());
                        }
                    }
                }

            }
        }

        @Override
        public void onInvitedNotify(@NotNull NotifyInvitedInfo notifyInfo) {

        }

        @Override
        public void onRefusedInviteNotify(@NotNull NotifyRefuseInviteInfo notifyInfo) {

        }

        @Override
        public void onRoomCloseNotify() {

        }

        @Override
        public void onRoomReconnected() {

        }

        @Override
        public void onUserVolumeUpdate(HashMap<Long, Float> userVolumeMap) {

        }

    };
}