package im.zego.gochat.chatroom.adapter;

import android.graphics.drawable.Drawable;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import im.zego.gochat.R;
import im.zego.gochat.chatroom.ChatRoomManager;
import im.zego.gochat.model.UserInfo;
import im.zego.gochat.utils.GlideImageLoader;
import im.zego.gochat.utils.MiscUtils;

/**
 * 房间内嘉宾
 */
public class ChatRoomStageGuestAdapter extends RecyclerView.Adapter<ChatRoomStageGuestAdapter.GuestViewHolder> {
    private UserInfoOrderedSet guestList = new UserInfoOrderedSet(UserInfoOrderedSet.ORDER_BY_ONSTAGE);
    private HashMap<Long, Float> userVolumeMap = new HashMap<>();
    private IAvatarClickListener mListener;

    public ChatRoomStageGuestAdapter(IAvatarClickListener listener) {
        this.mListener = listener;
    }

    @NotNull
    @Override
    public GuestViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_guest_list, parent, false);
        return new GuestViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull GuestViewHolder holder, int position) {
        UserInfo stageGuestUserInfo = guestList.get(position);
        GlideImageLoader.getInstance().displayImageByAvatar(stageGuestUserInfo.getAvatar(), holder.avatar);
        holder.userName.setText(stageGuestUserInfo.getNickName());

        Float volume = userVolumeMap.get(stageGuestUserInfo.getUserID());
        if (stageGuestUserInfo.isMicOpen() && volume != null && volume > 1) {
            holder.avatarTalking.setVisibility(View.VISIBLE);
        } else {
            holder.avatarTalking.setVisibility(View.INVISIBLE);
        }
        if (stageGuestUserInfo.isMicOpen()) {
            holder.mic.setVisibility(View.INVISIBLE);
        } else {
            holder.mic.setVisibility(View.VISIBLE);
        }

        if (stageGuestUserInfo.isHost()) {
            Drawable drawable = holder.userName.getContext().getResources().getDrawable(R.mipmap.host);
            drawable.setBounds(0, 0, drawable.getMinimumWidth(), drawable.getMinimumHeight());
            holder.userName.setCompoundDrawablePadding(MiscUtils.dp2px(holder.userName.getContext(),3));
            holder.userName.setCompoundDrawables(drawable, null, null, null);
        } else {
            holder.userName.setCompoundDrawables(null, null, null, null);
        }

        holder.itemView.setOnClickListener(v -> {
            UserInfo myUserInfo = ChatRoomManager.getInstance().getMyUserInfo();
            UserInfo clickedUserInfo = guestList.get(holder.getLayoutPosition());
            // 可以操作自己，但只有主持人才能操作别人，主持人不能操作自己
            if (myUserInfo.isHost()) {
                if (myUserInfo.getUserID() != clickedUserInfo.getUserID()) {
                    mListener.clickStageGuest(clickedUserInfo);
                }
            } else if (myUserInfo.getUserID() == clickedUserInfo.getUserID()) {
                mListener.clickSelf(clickedUserInfo);
            }
        });
    }

    @Override
    public int getItemCount() {
        return guestList.size();
    }


    /**
     * 初始化嘉宾列表
     */
    public void initUserList() {
        guestList.clear();
        Map<Long, UserInfo> userModelMap = ChatRoomManager.getInstance().getUserModelMap();
        Iterator iter = userModelMap.entrySet().iterator();
        while (iter.hasNext()) {
            Map.Entry entry = (Map.Entry) iter.next();
            UserInfo userModel = (UserInfo) entry.getValue();
            if (userModel.isStageGuest()) {
                guestList.add(userModel);
            } else if (userModel.isHost()) {
                guestList.add(0, userModel); //主持人放在首位
            }
        }
        notifyDataSetChanged();
    }

    public void onUserVolumeUpdated(HashMap<Long, Float> userVolumeMap) {
        this.userVolumeMap.putAll(userVolumeMap);

        for (Long uid : userVolumeMap.keySet()) {
            int idx = guestList.indexOf(uid);
            if (idx >= 0) {
                notifyItemChanged(idx);
            }
        }
    }

    /**
     * 嘉宾上台
     *
     * @param userInfo
     */
    public void onUserOnStage(UserInfo userInfo) {
        guestList.add(userInfo);
        notifyDataSetChanged();
    }

    /**
     * 嘉宾下台或者离开
     */
    public void onUserOffStage(long userID) {
        guestList.remove(userID);
        notifyDataSetChanged();
    }

    public void onUserMicStatusChanged(UserInfo changedUserInfo) {
        UserInfo updatedUserInfo = guestList.update(changedUserInfo);
        if (updatedUserInfo == null) {
            // 没有找到更新的，说明是新的用户，走新增流程
            onUserOnStage(changedUserInfo);
        }
        if(!changedUserInfo.isMicOpen()){
            userVolumeMap.remove(changedUserInfo.getUserID());
        }
        int idx = guestList.indexOf(updatedUserInfo);
        if (idx >= 0) {
            notifyItemChanged(idx);
        }
    }

    static class GuestViewHolder extends RecyclerView.ViewHolder {
        private ImageView avatarTalking;
        private ImageView avatar;
        private TextView userName;
        private ImageView mic;

        public GuestViewHolder(View itemView) {
            super(itemView);
            avatar = itemView.findViewById(R.id.iv_avatar);
            avatarTalking = itemView.findViewById(R.id.iv_avatar_talking);
            userName = itemView.findViewById(R.id.tv_user_name);
            mic = itemView.findViewById(R.id.iv_mic);
        }
    }
}
