package im.zego.gochat.custom;

import android.content.DialogInterface;
import android.graphics.Point;
import android.os.Bundle;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;

import im.zego.gochat.R;


/**
 * 默认点击外部区域收起，按返回键收起
 *
 * 注意如果里面的控件会恢复上次显示时候的状态,使用
 * view.isSaveFromParentEnabled 关闭
 */
public abstract class ZegoBaseDialogFragment extends DialogFragment {
    private ZegoBaseDialogFragment.OnDismissListener dismissListener;
    // isadd 和 isShowing 判断显示都有问题
    private boolean showedBySelf;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setStyle(DialogFragment.STYLE_NO_TITLE, getDialogStyle());
    }

    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        Window window = getDialog().getWindow();
        WindowManager.LayoutParams lp = window.getAttributes();
        lp.gravity = getGravity();
        window.setAttributes(lp);
        return createView(inflater, container);
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        if (getDialog() != null) {
            getDialog().setCanceledOnTouchOutside(canceledOnTouchOutside());
            getDialog().setCancelable(cancelable());
        }

    }

    @Override
    public void onStart() {
        super.onStart();
        if (getDialog() != null && getDialog().getWindow() != null) {
            Window window = getDialog().getWindow();
            Point dialogSize = getDialogSize();
            if (dialogSize != null) {
                window.setLayout(dialogSize.x, dialogSize.y);
            }
            window.setDimAmount(backgroundDimAmount());
//            window.addFlags(WindowManager.LayoutParams.FLAG_DIM_BEHIND);
        }
    }

    @Override
    public int show(@NonNull FragmentTransaction transaction, @Nullable String tag) {
        if (isAdded()) {
            return 0;
        }
        showedBySelf = true;
        try {
            return super.show(transaction, tag);
        } catch (Exception e) {
            transaction.add(this, tag);
            return transaction.commitAllowingStateLoss();
        }
    }

    @Override
    public void show(@NonNull FragmentManager manager, @Nullable String tag) {
        if (isFragmentAdded(manager)) {
            return;
        }
        showedBySelf = true;
        try {
            super.show(manager, tag);
        } catch (Exception e) {
            FragmentTransaction ft = manager.beginTransaction();
            ft.add(this, tag);
            ft.commitAllowingStateLoss();
        }
    }

    @Override
    public void showNow(@NonNull FragmentManager manager, @Nullable String tag) {
        if (isFragmentAdded(manager)) {
            return;
        }
        showedBySelf = true;
        try {
            super.showNow(manager, tag);
        } catch (Exception e) {
            FragmentTransaction ft = manager.beginTransaction();
            ft.add(this, tag);
            ft.commitNowAllowingStateLoss();
        }
    }

    /**
     * @param manager
     * @return
     */
    private boolean isFragmentAdded(@NonNull FragmentManager manager) {
        return isAdded() || manager.getFragments().contains(this) || showedBySelf;
    }

    @Override
    public void dismiss() {
        if (showedBySelf) {
            dismissAllowingStateLoss();
        }
    }

    protected boolean canceledOnTouchOutside() {
        return true;
    }

    protected boolean cancelable() {
        return true;
    }

    /**
     * @return 对话框的位置 Gravity.CENTER | Gravity.Bottom 等到
     */
    protected int getGravity() {
        return Gravity.CENTER;
    }

    protected float backgroundDimAmount() {
        return 0.15f;
    }

    /**
     * @return 需要特别的动画可以在这里设置style
     */
    protected int getDialogStyle() {
        return R.style.DialogFragmentStyle;
    }

    protected abstract View createView(LayoutInflater inflater, ViewGroup container);

    /**
     * @return 返回null表示使用默认的大小，否则使用返回的宽高，可以是 WRAP_CONTENT 或者是 MATCH_PARENT
     */
    protected Point getDialogSize() {
        return null;
    }

    public boolean isShowing() {
        return showedBySelf;
    }

    @Override
    public void onDismiss(DialogInterface dialog) {
        super.onDismiss(dialog);
        showedBySelf = false;
        if (dismissListener != null) {
            dismissListener.onDismiss();
        }
    }

    public void setOnDismissListener(OnDismissListener dismissListener) {
        this.dismissListener = dismissListener;
    }

    public interface OnDismissListener {
        void onDismiss();
    }
}
