package im.zego.gochat.protocol;

import android.util.Log;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import im.zego.gochat.chatroom.ChatRoomManager;
import im.zego.gochat.model.NoInfo;
import im.zego.gochat.model.RaiseHandUserInfoData;
import im.zego.gochat.model.RoomInfo;
import im.zego.gochat.model.RoomListInfo;
import im.zego.gochat.model.UserInfoData;
import im.zego.gochat.rtc.RTCSDKManager;
import im.zego.gochat.rtc.callbacks.IRTCCommonCallback;

public class RoomAPI {
    private static final String TAG = "RoomAPI";

    /**
     * 拉取房间列表
     *
     * @param uid
     * @param count
     * @param beginTimestamp
     * @param callback
     */
    public static void getRoomList(long uid, int count, long beginTimestamp, @NotNull final IGetRoomListCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("count", count)
                .addParams("begin_timestamp", beginTimestamp)
                .build();

        APIBase.asyncPost("/chat_room/get_room_list", reqJson, RoomListInfo.class, new IAsyncPostCallback<RoomListInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, RoomListInfo responseJsonBean) {
                if (callback != null) {
                    callback.onGetRoomList(retcode, responseJsonBean);
                }
            }
        });
    }

    /**
     * 登录房间
     *
     * @param uid
     * @param room_id
     * @param callback
     */
    public static void loginRoom(long uid, String room_id, @NotNull final ILoginRoomCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", room_id)
                .build();

        APIBase.asyncPost("/chat_room/login_room", reqJson, RoomInfo.class, new IAsyncPostCallback<RoomInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, RoomInfo roomInfo) {
                if (errorCode == 0) {
                    RTCSDKManager.getInstance().loginRoom(roomInfo.getRoomID(), new IRTCCommonCallback() {
                        @Override
                        public void onRTCCallback(int errorCode) {
                            Log.d(TAG, "rtc login_room errorCode:" + errorCode);
                        }
                    });
                }

                callback.onLoginRoom(errorCode, roomInfo);
            }
        });
    }

    /***
     * 创建房间
     *
     * @param uid
     * @param subject
     * @param callback
     */
    public static void createRoom(long uid, String subject, @NotNull final ICreateRoomCallback callback)
    {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("subject", subject)
                .build();

        APIBase.asyncPost("/chat_room/create_room",reqJson, RoomInfo.class, (errorCode, message, roomInfo) -> {
            if (errorCode == 0) {
                RTCSDKManager.getInstance().loginRoom(roomInfo.getRoomID(), new IRTCCommonCallback() {
                    @Override
                    public void onRTCCallback(int errorCode) {
                        Log.d(TAG, "rtc login_room errorCode:" + errorCode);
                    }
                });
            }
            callback.onCreateRoom(errorCode, roomInfo);
        });
    }



    /**
     * 获取用户列表
     *
     * @param uid
     * @param page
     * @param pageSize
     * @param callback
     */
    public static void getUserList(long uid, int page, int pageSize, @NotNull final IGetUserListCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", ChatRoomManager.getInstance().getRoomInfo().getRoomID())
                .addParams("page", page)
                .addParams("page_size", pageSize)
                .build();

        APIBase.asyncPost("/chat_room/get_attendee_list", reqJson, UserInfoData.class, new IAsyncPostCallback<UserInfoData>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, UserInfoData responseJsonBean) {
                if (callback != null) {
                    callback.onGetUserList(retcode, responseJsonBean);
                }
            }
        });
    }


    /**
     * 获取举手列表
     *
     * @param uid
     * @param page
     * @param pageSize
     * @param callback
     */
    public static void getRaiseHandList(long uid, int page, int pageSize, @NotNull final IGetRaiseHandUserListCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", ChatRoomManager.getInstance().getRoomInfo().getRoomID())
                .addParams("page", page)
                .addParams("page_size", pageSize)
                .build();

        APIBase.asyncPost("/chat_room/get_raise_hand_list", reqJson, RaiseHandUserInfoData.class, new IAsyncPostCallback<RaiseHandUserInfoData>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, RaiseHandUserInfoData responseJsonBean) {
                if (callback != null) {
                    callback.onGetRaiseHandUserList(retcode, responseJsonBean);
                }
            }
        });
    }

    /***
     * 离开房间
     *
     * @param uid
     * @param callback
     */
    public static void quitRoom(long uid, @Nullable final ICommonResponseCallback callback)
    {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", ChatRoomManager.getInstance().getRoomInfo().getRoomID())
                .build();

        APIBase.asyncPost("/chat_room/quit_room", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }


    /**
     * 关闭房间
     *
     * @param uid
     * @param callback
     */
    public static void closeRoom(long uid, @NotNull final ICommonResponseCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", ChatRoomManager.getInstance().getRoomInfo().getRoomID())
                .build();

        APIBase.asyncPost("/chat_room/close_room", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }


    /**
     * 举手
     *
     * @param uid
     * @param callback
     */
    public static void raiseHand(long uid, boolean isRaiseHand, @NotNull final ICommonResponseCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("type", isRaiseHand ? 2 : 1)
                .addParams("room_id", ChatRoomManager.getInstance().getRoomInfo().getRoomID())
                .build();

        APIBase.asyncPost("/chat_room/operate_raise_hand", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 邀请上台
     *
     * @param uid
     * @param callback
     */
    public static void inviteOnstage(long uid,long targetUid, @NotNull final ICommonResponseCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("target_uid",targetUid)
                .addParams("room_id", ChatRoomManager.getInstance().getRoomInfo().getRoomID())
                .build();

        APIBase.asyncPost("/chat_room/invite_onstage", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 回应邀请
     *
     * @param uid
     * @param callback
     */
    public static void responseinviteOnstage(long uid,String token,int respond, @NotNull final ICommonResponseCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("invite_token",token)
                .addParams("respond",respond)
                .addParams("room_id", ChatRoomManager.getInstance().getRoomInfo().getRoomID())
                .build();

        APIBase.asyncPost("/chat_room/response_onstage_invite", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }


    /**
     * 设置用户状态
     *
     * @param uid
     * @param callback
     * @type 1、麦克风操作 2、设置为台下观众
     */
    public static void setUserInfo(long uid, long targetUid, boolean isMicOpen, int onstage, int type, @NotNull final ICommonResponseCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", ChatRoomManager.getInstance().getRoomInfo().getRoomID())
                .addParams("target_uid", targetUid)
                .addParams("mic", isMicOpen ? 2 : 1)
                .addParams("on_stage", onstage)
                .addParams("type",type)
                .build();

        APIBase.asyncPost("/chat_room/set_user_info", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }
}
