package im.zego.ktv.chorus.ktvroom;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkRequest;

import androidx.fragment.app.FragmentActivity;

import java.util.concurrent.atomic.AtomicBoolean;

import im.zego.common.util.SafetyHandler;
import im.zego.ktv.chorus.ktvroom.callback.INetworkKtvCallback;
import im.zego.ktv.chorus.login.LoginInfoManager;
import im.zego.ktv.chorus.utils.NetworkKtvCallbackImpl;

/**
 * ktv 网络管理
 */
public class NetWorkKtvConManager {

    private static final String TAG = "NetWorkConManager";

    private static final long DEFAULT_DELAY_TIME = 90 * 1000; //延时执行时间 90s

    private INetworkKtvCallback mCallback;
    private ConnectivityManager connMgr;
    private NetworkKtvCallbackImpl networkCallback;

    /**
     * 是否是第一次开始连接网咯
     */
    private AtomicBoolean startFirst = new AtomicBoolean(true);

    /**
     * 心跳断网标志
     */
    private AtomicBoolean heartBeatDisconnect = new AtomicBoolean(true);

    /**
     * 定时任务
     */
    private SafetyHandler safetyHandler;


    private volatile static NetWorkKtvConManager mInstance;

    public static NetWorkKtvConManager getInstance() {
        if (mInstance == null) {
            synchronized (NetWorkKtvConManager.class) {
                if (mInstance == null) {
                    mInstance = new NetWorkKtvConManager();
                }
            }
        }
        return mInstance;
    }

    /**
     * 网咯监听-有网
     *
     * @param isAvailable
     */
    public void networkOnAvailable(boolean isAvailable) {
        if (isAvailable && !startFirst.get()) {
            setStartFirst(false);
            if (safetyHandler != null) {
                safetyHandler.removeCallbacks(delayRunnable);
            }
            if (mCallback != null) {
                mCallback.onStageList();
            }
        }

        if (startFirst.get()) {
            setStartFirst(false);
        }
    }

    /**
     * 网咯监听-没网
     */
    public void networkOnLost() {
        if (mCallback != null) {
            mCallback.networkOnLost();
        }
        if (safetyHandler != null) {
            safetyHandler.removeCallbacks(delayRunnable);
            safetyHandler.postDelayed(delayRunnable, DEFAULT_DELAY_TIME);
        }
    }

    /**
     * 断网90s延时执行-网络异常弹窗
     */
    private Runnable delayRunnable = () -> {
        if (mCallback != null) {
            mCallback.networkOnLostCloseRoom();
        }
    };

    /**
     * 心跳返回成功
     */
    public void onHeartBeatConnect() {
        if (!heartBeatDisconnect.get()) {
            heartBeatDisconnect.set(true);
            if (safetyHandler != null) {
                safetyHandler.removeCallbacks(delayHeartBeatRunnable);
            }

            if (mCallback != null) {
                mCallback.onStageList();
            }
        }
    }


    /**
     * 心跳断开连接
     */
    public void onHeartBeatDisconnect() {
        if (safetyHandler != null && heartBeatDisconnect.get()) {
            if (mCallback != null) {
                mCallback.networkOnLost();
            }
            heartBeatDisconnect.set(false);
            safetyHandler.removeCallbacks(delayHeartBeatRunnable);
            safetyHandler.postDelayed(delayHeartBeatRunnable, DEFAULT_DELAY_TIME);
        }
    }

    /**
     * 心跳断开90s延时执行-网络异常弹窗
     */
    private Runnable delayHeartBeatRunnable = () -> {
        if (mCallback != null) {
            mCallback.networkOnLostCloseRoom();
        }
    };

    /**
     * 关闭房间
     */
    public void onCloseRoom() {
        if (mCallback != null) {
            mCallback.networkOnLostCloseRoom();
        }
    }

    /**
     * 注册网络状态的回调
     *
     * @param context
     */
    public void registerNetworkCallback(Context context) {
        networkCallback = new NetworkKtvCallbackImpl();
        NetworkRequest.Builder builder = new NetworkRequest.Builder();
        NetworkRequest request = builder.build();
        connMgr = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        if (connMgr != null) {
            connMgr.registerNetworkCallback(request, networkCallback);
        }

        safetyHandler = SafetyHandler.with((FragmentActivity) context);

        LoginInfoManager.getInstance().setInRoom(true);
    }

    private void unregisterNetworkCallback() {
        if (connMgr != null && networkCallback != null) {
            connMgr.unregisterNetworkCallback(networkCallback);
        }
    }

    public void setCallBack(INetworkKtvCallback callBack) {
        this.mCallback = callBack;
    }

    public void setStartFirst(boolean flag) {
        this.startFirst.set(flag);
    }

    public void clearData() {
        setStartFirst(true);
        heartBeatDisconnect.set(true);
        setCallBack(null);
        LoginInfoManager.getInstance().setInRoom(false);
        unregisterNetworkCallback();
        if (safetyHandler != null) {
            safetyHandler.onDestroy();
        }
    }

}
