package im.zego.ktv.chorus.login;

import static im.zego.ktv.ErrorCodeConstants.ERROR_USER_OFF_LINE;

import android.os.Handler;
import android.os.Looper;

import org.jetbrains.annotations.NotNull;

import java.util.Timer;
import java.util.TimerTask;

import im.zego.common.model.LoginInfo;
import im.zego.common.util.SharedPreferencesUtil;
import im.zego.ktv.chorus.ktvroom.NetWorkKtvConManager;
import im.zego.ktv.chorus.model.HeartbeatRespInfo;
import im.zego.ktv.chorus.protocol.ICommonResponseCallback;
import im.zego.ktv.chorus.protocol.IHeartBeatCallback;
import im.zego.ktv.chorus.protocol.ILoginCallback;
import im.zego.ktv.chorus.protocol.LoginAPI;

public class LoginInfoManager {

    private static volatile LoginInfoManager singleton = null;

    public static LoginInfoManager getInstance() {
        if (singleton == null) {
            synchronized (LoginInfoManager.class) {
                if (singleton == null) {
                    singleton = new LoginInfoManager();
                    sMainHandler = new Handler(Looper.getMainLooper());
                }
            }
        }
        return singleton;
    }

    private TimerTask sHeartBeatTimerTask;
    private long heartInterval = 30;
    private Timer mHeartTimer;
    private static Handler sMainHandler;
    private boolean isInRoom = false;

    public void setLoginInfo(LoginInfo loginInfo) {
        SharedPreferencesUtil.cacheLoginInfo(loginInfo);
    }

    public LoginInfo getLoginInfo() {
        return SharedPreferencesUtil.getLoginInfo();
    }

    public boolean isLogin() {
        return SharedPreferencesUtil.getLoginInfo().getUid() != 0;
    }

    public void clearLoginInfo() {
        SharedPreferencesUtil.clearLoginInfo();
    }

    public void loginUser(@NotNull String nickName, @NotNull String avatar, @NotNull final ILoginCallback<LoginInfo> callback) {
        LoginAPI.login(nickName, avatar, new ILoginCallback<LoginInfo>() {
            @Override
            public void onLogin(int errorCode, LoginInfo loginInfo) {
                if (errorCode == 0) {

                    if (mHeartTimer != null && sHeartBeatTimerTask != null) {
                        sHeartBeatTimerTask.cancel();
                        mHeartTimer.cancel();
                        mHeartTimer = null;
                        sHeartBeatTimerTask = null;
                    }

                    heartBeat(loginInfo.getUid());

                }
                callback.onLogin(errorCode, loginInfo);
            }
        });
    }

    public void logoutUser(ICommonResponseCallback callback) {
        if (getLoginInfo() != null) {
            LoginAPI.logout(getLoginInfo().getUid(), new ICommonResponseCallback() {
                @Override
                public void onResponse(int errorCode) {
                    if (callback != null) {
                        callback.onResponse(errorCode);
                    }

                    if (mHeartTimer != null) {
                        sHeartBeatTimerTask.cancel();
                        mHeartTimer.cancel();
                        mHeartTimer = null;
                        sHeartBeatTimerTask = null;
                    }
                }
            });
            LoginInfoManager.getInstance().clearLoginInfo();
        }
    }

    /**
     * 心跳
     *
     * @param uid
     */
    public void heartBeat(long uid) {
        if (mHeartTimer == null) {
            mHeartTimer = new Timer();
            sHeartBeatTimerTask = new HeartBeatTimerTask();
            LoginAPI.heartBeat(uid, new IHeartBeatCallback() {
                @Override
                public void onHeartBeat(int errorCode, HeartbeatRespInfo heartbeatRespInfo) {
                    //心跳依赖于第一次成功，然后 heartbeatRespInfo.getInterval() - 5 ，之后都是这个时间的定时器执行
                    if (errorCode == 0) {
                        heartInterval = heartbeatRespInfo.getInterval();
                        if (heartInterval > 5) {
                            heartInterval -= 5;
                        }
                        mHeartTimer.schedule(sHeartBeatTimerTask, heartInterval * 1000, heartInterval * 1000);
                    }
                }
            });
        }
    }

    private class HeartBeatTimerTask extends TimerTask {
        @Override
        public void run() {

            sMainHandler.post(() -> {
                LoginAPI.heartBeat(LoginInfoManager.getInstance().getLoginInfo().getUid(), new IHeartBeatCallback() {
                    @Override
                    public void onHeartBeat(int errorCode, HeartbeatRespInfo heartbeatRespInfo) {
                        if (isInRoom()) {
                            if (errorCode == 0) {
                                NetWorkKtvConManager.getInstance().onHeartBeatConnect();
                            } else {
                                if (errorCode == ERROR_USER_OFF_LINE) { //用户不在线关闭该房间
                                    NetWorkKtvConManager.getInstance().onCloseRoom();
                                } else {
                                    NetWorkKtvConManager.getInstance().onHeartBeatDisconnect();
                                }
                            }
                        }
                    }
                });
            });

        }
    }


    public boolean isInRoom() {
        return isInRoom;
    }

    public void setInRoom(boolean inRoom) {
        isInRoom = inRoom;
    }
}
