package im.zego.ktv.chorus.protocol;

import static im.zego.ktv.ErrorCodeConstants.ERROR_ROOM_INFO_NULL;

import android.util.Log;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import im.zego.ktv.chorus.ktvroom.KTVRoomManager;
import im.zego.ktv.chorus.model.NoInfo;
import im.zego.ktv.chorus.model.OrderSongInfo;
import im.zego.ktv.chorus.model.OrderedSongInfo;
import im.zego.ktv.chorus.model.RoomInfo;
import im.zego.ktv.chorus.model.RoomListInfo;
import im.zego.ktv.chorus.model.UserInfo;
import im.zego.ktv.chorus.model.UserInfoData;
import im.zego.ktv.chorus.model.notify.NotifyRoomStateUpdateInfo;
import im.zego.ktv.chorus.rtc.RTCSDKManager;
import im.zego.ktv.chorus.rtc.callbacks.IRTCCommonCallback;

public class RoomAPI {
    private static final String TAG = "RoomAPI";

    /**
     * 拉取房间列表
     *
     * @param uid
     * @param count
     * @param beginTimestamp
     * @param callback
     */
    public static void getRoomList(long uid, int count, long beginTimestamp, @NotNull final IGetRoomListCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("count", count)
                .addParams("begin_timestamp", beginTimestamp)
                .build();

        APIBase.asyncPost("/ktv_room/get_room_list", reqJson, RoomListInfo.class, new IAsyncPostCallback<RoomListInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, RoomListInfo responseJsonBean) {
                if (callback != null) {
                    callback.onGetRoomList(retcode, responseJsonBean);
                }
            }
        });
    }


    /***
     * 创建房间
     *
     * @param uid
     * @param subject
     * @param callback
     */
    public static void createRoom(long uid,String nickName,String avatar, String subject,String coverImg, @NotNull final ICreateRoomCallback callback)
    {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("nick_name", nickName)
                .addParams("avatar", avatar)
                .addParams("subject", subject)
                .addParams("cover_img", coverImg)
                .build();

        APIBase.asyncPost("/ktv_room/create_room",reqJson, NotifyRoomStateUpdateInfo.class, (errorCode, message, roomInfo) -> {
            if (errorCode == 0) {
                RTCSDKManager.getInstance().setHost(roomInfo.getCreatorId());
                RTCSDKManager.getInstance().loginRoom(roomInfo.getRoomId(), new IRTCCommonCallback() {
                    @Override
                    public void onRTCCallback(int errorCode) {
                        Log.d(TAG, "rtc login_room errorCode:" + errorCode);
                    }
                });
            }
            callback.onCreateRoom(errorCode, roomInfo);
        });
    }



    /**
     * 进入房间
     *
     * @param uid
     * @param room_id
     * @param callback
     */
    public static void loginRoom(long uid, String room_id,String nickName,String avatar, @NotNull final ILoginRoomCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", room_id)
                .addParams("nick_name", nickName)
                .addParams("avatar", avatar)
                .build();

        APIBase.asyncPost("/ktv_room/login_room", reqJson, NotifyRoomStateUpdateInfo.class, new IAsyncPostCallback<NotifyRoomStateUpdateInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, NotifyRoomStateUpdateInfo notifyRoomStateUpdateInfo) {
                if (errorCode == 0) {
                    RTCSDKManager.getInstance().loginRoom(notifyRoomStateUpdateInfo.getRoomId(), new IRTCCommonCallback() {
                        @Override
                        public void onRTCCallback(int errorCode) {
                            Log.d(TAG, "rtc login_room errorCode:" + errorCode);
                        }
                    });
                }

                callback.onLoginRoom(errorCode, notifyRoomStateUpdateInfo);
            }
        });
    }


    /***
     * 离开房间
     *
     * @param uid
     * @param callback
     */
    public static void quitRoom(long uid, @Nullable final ICommonResponseCallback callback)
    {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .build();

        APIBase.asyncPost("/ktv_room/quit_room", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }


    /**
     * 关闭房间
     *
     * @param uid
     * @param callback
     */
    public static void closeRoom(long uid, @NotNull final ICommonResponseCallback callback) {

        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .build();

        APIBase.asyncPost("/ktv_room/close_room", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 拉取已点歌曲列表
     *
     * @param uid
     * @param callback
     */
    public static void getOrderedSongList(long uid,  @NotNull final IGetOrderedSongListCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onGetOrderedSongList(ERROR_ROOM_INFO_NULL,null);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("uid", uid)
                .build();

        APIBase.asyncPost("/ktv_room/get_ordered_songs_list", reqJson, OrderedSongInfo.class, new IAsyncPostCallback<OrderedSongInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, OrderedSongInfo responseJsonBean) {
                if (callback != null) {
                    callback.onGetOrderedSongList(retcode, responseJsonBean);
                }
            }
        });
    }



    /**
     * 拉取台上用户列表
     *
     * @param uid
     * @param callback
     */
    public static void getUserList(long uid, @NotNull final IGetUserListCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onGetUserList(ERROR_ROOM_INFO_NULL,null);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .build();

        APIBase.asyncPost("/ktv_room/get_onstage_attendee_list", reqJson, UserInfoData.class, new IAsyncPostCallback<UserInfoData>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, UserInfoData responseJsonBean) {
                if (callback != null) {
                    callback.onGetUserList(retcode, responseJsonBean);
                }
            }
        });
    }


    /**
     * 设置mic状态
     * @param uid
     * @param targetUid
     * @param mic
     * @param callback
     */
    public static void setMicStatus(long uid, long targetUid, int mic, @NotNull final ICommonResponseCallback callback) {

        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("target_uid", targetUid)
                .addParams("mic", mic)
                .addParams("type", 1)
                .build();

        APIBase.asyncPost("/ktv_room/set_status", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 设置上下台状态
     * @param uid
     * @param targetUid
     * @param stageState
     * @param micIndex
     * @param type 2、设置为台下观众 3.设置为上台（需要带上mic_index)
     * @param callback
     */
    public static void setStageStatus(long uid, long targetUid, int stageState, int micIndex, int type, @NotNull final ICommonResponseCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("target_uid", targetUid)
                .addParams("onstage_state", stageState)
                .addParams("mic_index", micIndex)
                .addParams("type", type)
                .build();

        APIBase.asyncPost("/ktv_room/set_status", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }


    /**
     * 设置为台下观众
     * @param uid
     * @param targetUid
     * @param callback
     */
    public static void setOffStage(long uid, long targetUid, @NotNull final ICommonResponseCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("target_uid", targetUid)
                .addParams("onstage_state", 1)
                .addParams("type", 2)
                .build();

        APIBase.asyncPost("/ktv_room/set_status", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 设置为上台（需要带上mic_index)
     * @param uid
     * @param targetUid
     * @param micIndex
     * @param callback
     */
    public static void setOnStage(long uid, long targetUid, int micIndex, @NotNull final ICommonResponseCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("target_uid", targetUid)
                .addParams("onstage_state", 2)
                .addParams("mic_index", micIndex)
                .addParams("type", 3)
                .build();

        APIBase.asyncPost("/ktv_room/set_status", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 切哥和取消点歌
     * @param uid
     * @param targetUid
     * @param targetNickName
     * @param songId
     * @param type
     * @param songCreateTime
     * @param callback
     */
    public static void setSongStatus(long uid, long targetUid, String targetNickName, int songId, int type, long songCreateTime, @NotNull final ICommonResponseCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("target_uid", targetUid)
                .addParams("target_nick_name", targetNickName)
                .addParams("song_create_time", songCreateTime)
                .addParams("song_id", songId)
                .addParams("type", type)
                .build();

        APIBase.asyncPost("/ktv_room/set_status", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 原唱/伴唱 设置
     * @param uid
     * @param targetUid
     * @param targetNickName
     * @param originPlay 原唱为true,伴奏为false
     * @param songId
     * @param songCreateTime
     * @param callback
     */
    public static void setSwitchAudioTrack(long uid, long targetUid, String targetNickName, boolean originPlay,String songId, long songCreateTime, @NotNull final ICommonResponseCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("target_uid", targetUid)
                .addParams("origin_play",originPlay?1:0)
                .addParams("target_nick_name", targetNickName)
                .addParams("song_create_time", songCreateTime)
                .addParams("song_id", songId)
                .addParams("type", 4)
                .build();

        APIBase.asyncPost("/ktv_room/set_status", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 切歌
     * @param uid
     * @param targetUid
     * @param targetNickName
     * @param songId
     * @param songCreateTime
     * @param callback
     */
    public static void setSkipSong(long uid, long targetUid, String targetNickName,int songId, long songCreateTime, @NotNull final ICommonResponseCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("target_uid", targetUid)
                .addParams("target_nick_name", targetNickName)
                .addParams("song_create_time", songCreateTime)
                .addParams("song_id", songId)
                .addParams("type", 5)
                .build();

        APIBase.asyncPost("/ktv_room/set_status", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 取消点歌
     * @param uid
     * @param targetUid
     * @param targetNickName
     * @param songCreateTime
     * @param callback
     */
    public static void setCancelSong(long uid, long targetUid, String targetNickName, long songCreateTime,String songUniId, @NotNull final ICommonResponseCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("target_uid", targetUid)
                .addParams("target_nick_name", targetNickName)
                .addParams("song_create_time", songCreateTime)
                .addParams("song_uni_id", songUniId)
                .addParams("type", 7)
                .build();

        APIBase.asyncPost("/ktv_room/set_status", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 置顶歌曲
     * @param uid
     * @param targetUid
     * @param targetNickName
     * @param songCreateTime
     * @param callback
     */
    public static void setTopSong(long uid, long targetUid, String targetNickName, long songCreateTime,String songUniId, @NotNull final ICommonResponseCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("target_uid", targetUid)
                .addParams("target_nick_name", targetNickName)
                .addParams("song_create_time", songCreateTime)
                .addParams("song_uni_id", songUniId)
                .addParams("type", 8)
                .build();

        APIBase.asyncPost("/ktv_room/set_status", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 获取房间信息
     * @param uid
     * @param callback
     */
    public static void getRoomInfo(long uid, @NotNull final IGetRoomInfoCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onGetRoomInfo(ERROR_ROOM_INFO_NULL,null);
            }
            return;
        }
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .build();

        APIBase.asyncPost("/ktv_room/get_room_info", reqJson, RoomInfo.class, new IAsyncPostCallback<RoomInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, RoomInfo responseJsonBean) {
                if (callback != null) {
                    callback.onGetRoomInfo(retcode,responseJsonBean);
                }
            }
        });
    }


    /**
     * 透传广播，目前只用于通知歌曲在某个ntp时间点开始播放
     * @param uid
     * @param millis
     * @param type
     * @param callback
     */
    public static void setBroadcastMsg(long uid, long millis, int type, @NotNull final ICommonResponseCallback callback) {
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }

        String message = APIBase.builder()
                .addParams("time", millis)
                .addParams("type", type)
                .build();

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("message", message)
                .build();

        APIBase.asyncPost("/ktv_room/broadcast_message", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }


    /**
     * 点歌
     * @param uid
     * @param callback
     */
    public static void orderSong(long uid, String songId, String shareToken, String krcToken, @NotNull final IOrderSongCallback callback) {

        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onOrderSong(ERROR_ROOM_INFO_NULL,null);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("song_id",songId)
                .addParams("share_token",shareToken)
                .addParams("krc_token", krcToken)
                .build();

        APIBase.asyncPost("/ktv_room/order_song", reqJson, OrderSongInfo.class, new IAsyncPostCallback<OrderSongInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, OrderSongInfo responseJsonBean) {
                if (callback != null) {
                    callback.onOrderSong(retcode,responseJsonBean);
                }
            }
        });
    }

    /**
     * 点歌同步回复
     * @param uid
     * @param callback
     */
    public static void orderSongNotifyAck(long uid,String song_uni_id, @NotNull final ICommonResponseCallback callback) {

        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("song_uni_id",song_uni_id)
                .build();

        APIBase.asyncPost("/ktv_room/order_song_notify_ack", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }


    /**
     * 歌曲下载完成上报
     * @param uid
     * @param callback
     */
    public static void songDownloadReport(long uid,String song_uni_id ,@NotNull final ICommonResponseCallback callback) {

        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            if(callback != null){
                callback.onResponse(ERROR_ROOM_INFO_NULL);
            }
            return;
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", KTVRoomManager.getInstance().getRoomInfo().getRoomId())
                .addParams("song_uni_id",song_uni_id)
                .build();

        APIBase.asyncPost("/ktv_room/song_download_report", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }


    /**
     * 获取用户自身信息
     * @param uid
     * @param callback
     */
    public static void getUserInfo(long uid,String roomId, @NotNull final IGetUserInfoCallback callback) {


        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", roomId)
                .build();

        APIBase.asyncPost("/ktv_room/get_user_info", reqJson, UserInfo.class, new IAsyncPostCallback<UserInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, UserInfo responseJsonBean) {
                if (callback != null) {
                    callback.onGetUserInfo(retcode, responseJsonBean);
                }
            }
        });
    }



}
