package im.zego.lyricview;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.ValueAnimator;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.os.Looper;
import android.text.TextUtils;
import android.util.AttributeSet;

import android.util.TypedValue;
import android.view.View;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import im.zego.lyricview.listener.OnLyricLineFinishedListener;
import im.zego.lyricview.model.LineInfo;
import im.zego.lyricview.model.LyricInfo;
import im.zego.lyricview.model.WordInfo;
import im.zego.lyricview.model.ZGKTVLyric;

public class ZegoLyricView extends View {
    private final static String TAG = "ZegoLyricView";

    // 所有字体颜色
    private int mHintColor = Color.parseColor("#FFFFFF");  // 提示语颜色
    private int mDefaultColor = Color.parseColor("#CCFFFFFF");  // 默认字体颜色
    private int mHighLightColor = Color.parseColor("#ff3571");  // 当前播放位置的颜色
    private int mShaderColor = Color.parseColor("#FFFFFF");
    private int mShaderColorAlpha = 0x80;

    private LinesPattern mLinesPattern = LinesPattern.TEO_LINES;

    private int mLineCount;  // 行数
    // 所有的行高
    private float mHintTextHeight;  // 提示语行高
    private float mDefaultTextHeight; // 默认字体 行高
    private float mHighLightTextHeight;  // 高亮字体 行高
    private float mShaderTextHeight;  // 阴影字体行高

    // 字体大小
    private float mDefaultTextSizePx;
    private float mHighLightTextSizePx;
    private float mShaderTextSizePx;
    private float mHintTextSizePx;

    // 各TextView 偏移
    private float mDefaultTextPy = 0;
    private float mHighLightTextPy = 0;
    private float mShaderTextPy = 0;

    // 最底下一行透明色
    private int mBottomTextAlpha = 0;

    private float mLineSpace = 0;  // 行间距（包含在行高中）
    private int mCurrentPlayLine = -1;  // 当前播放位置对应的行数

    //设置长歌词从屏幕1/mScale处开始滚动到结束停止，当前默认为长歌词播放到1/3之一就开始滚动。
    private int mScale = 3;

    private volatile LyricInfo mLyricInfo;
    private String mDefaultHint = "ZegoLyricView";

    private Paint mHintTextPaint;          // 提示语字体画笔（中间）
    private Paint mDefaultTextPaint;       // 默认字体画笔
    private Paint mHighLightTextPaint;     // 高亮字体画笔（顶部）
    private Paint mShaderTextPaint;        // 阴影字体画笔（底下）
    private Paint mBottomTextPaint;        // 最底下字体画笔，仅出现在滚动的时候，颜色，大小跟上面mShaderTextPaint的一致

    private boolean mSliding = false;

    private static final long MAX_SMOOTH_SCROLL_DURATION = 300;
    /**
     * 歌词高亮的模式 逐行模式
     */
    private final int MODE_HIGH_LIGHT_NORMAL = 0;
    /**
     * 歌词高亮的模式 逐字模式
     */
    private final int MODE_HIGH_LIGHT_KRC = 1;

    /**
     * 歌词高亮的模式
     */
    private int mode = MODE_HIGH_LIGHT_KRC;

    private long mCurrentTimeMillis;
    // 歌词滚动动效
    private ValueAnimator valueAnimator;

    private final static int TOLERANCE_TIME = 20;
    private int mLineIntervalTime = 500;

    private final List<LyricLineFeed> mLineFeedList = new ArrayList<>();

    private OnLyricLineFinishedListener mOnLyricLineFinishedListener;

    private int mLineFeedPosition = -1;

    public ZegoLyricView(Context context) {
        super(context);
        initView();
    }

    public ZegoLyricView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initView();
    }

    public ZegoLyricView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initView();
    }

    private void initView() {
        float defaultTextSizeForPx = getRawSize(TypedValue.COMPLEX_UNIT_SP, 12);

        mDefaultTextSizePx = defaultTextSizeForPx;
        mHighLightTextSizePx = defaultTextSizeForPx;
        mShaderTextSizePx = defaultTextSizeForPx;
        mHintTextSizePx = defaultTextSizeForPx;

        initAllPaints();
        initAllBounds();
    }

    /**
     * 初始化画笔
     */
    private void initAllPaints() {
        mHintTextPaint = new Paint();
        mHintTextPaint.setDither(true);
        mHintTextPaint.setAntiAlias(true);
        mHintTextPaint.setTextAlign(Paint.Align.CENTER);
        mHintTextPaint.setColor(mHintColor);
        mHintTextPaint.setTextSize(mHintTextSizePx);

        mDefaultTextPaint = new Paint();
        mDefaultTextPaint.setDither(true);
        mDefaultTextPaint.setAntiAlias(true);
        mDefaultTextPaint.setTextAlign(Paint.Align.CENTER);
        mDefaultTextPaint.setColor(mDefaultColor);
        mDefaultTextPaint.setTextSize(mDefaultTextSizePx);

        mHighLightTextPaint = new Paint();
        mHighLightTextPaint.setDither(true);
        mHighLightTextPaint.setAntiAlias(true);
        mHighLightTextPaint.setTextAlign(Paint.Align.CENTER);
        mHighLightTextPaint.setColor(mHighLightColor);
        mHighLightTextPaint.setTextSize(mHighLightTextSizePx);

        mShaderTextPaint = new Paint();
        mShaderTextPaint.setDither(true);
        mShaderTextPaint.setAntiAlias(true);
        mShaderTextPaint.setTextAlign(Paint.Align.CENTER);
        mShaderTextPaint.setColor(mShaderColor);
        mShaderTextPaint.setTextSize(mShaderTextSizePx);

        mBottomTextPaint = new Paint();
        mBottomTextPaint.setDither(true);
        mBottomTextPaint.setAntiAlias(true);
        mBottomTextPaint.setTextAlign(Paint.Align.CENTER);
        // 颜色字体大小跟mShaderTextPaint的一致
        mBottomTextPaint.setColor(mShaderColor);
        mBottomTextPaint.setTextSize(mShaderTextSizePx);
        // 设置透明度
        mBottomTextPaint.setAlpha(mBottomTextAlpha);
    }

    /**
     * 初始化需要的尺寸
     */
    private void initAllBounds() {
        Rect lineBound = new Rect();
        mHintTextPaint.getTextBounds(mDefaultHint, 0, mDefaultHint.length(), lineBound);
        mHintTextHeight = lineBound.height();
        mDefaultTextPaint.getTextBounds(mDefaultHint, 0, mDefaultHint.length(), lineBound);
        mDefaultTextHeight = lineBound.height();
        mHighLightTextPaint.getTextBounds(mDefaultHint, 0, mDefaultHint.length(), lineBound);
        mHighLightTextHeight = lineBound.height();
        mShaderTextPaint.getTextBounds(mDefaultHint, 0, mDefaultHint.length(), lineBound);
        mShaderTextHeight = lineBound.height();
    }

    /**
     * 直接绘制三行，不管空间是否足够。。因此自己控制字体或者控件大小吧。。
     */
    @Override
    protected void onDraw(Canvas canvas) {
        if (mCurrentPlayLine < 0) {
            return;
        }
        final int height = getMeasuredHeight(); // height of this view
        final int width = getMeasuredWidth(); // width of this view
        final int paddingTop = getPaddingTop();

        if (mLyricInfo != null && mLyricInfo.song_lines != null && mLyricInfo.song_lines.size() > 0) {
            // 绘制头顶行
            if (mCurrentPlayLine - 1 >= 0 || mCurrentPlayLine == mLyricInfo.song_lines.size()) {
                if (mode == MODE_HIGH_LIGHT_NORMAL) {
                    // 逐行模式
                    drawHighLightLrcRow(canvas, mLyricInfo.song_lines.get(mCurrentPlayLine - 1).content,
                            width * 0.5f,
                            mHighLightTextHeight - mHighLightTextPy + paddingTop);
                } else {
                    // 逐字模式
                    float progress = ZegoLyricViewHelper.calculateCurrentKrcProgress(mCurrentTimeMillis, mLyricInfo.song_lines.get(mCurrentPlayLine - 1));
                    drawKaraokeHighLightLrcRow(canvas, mLyricInfo.song_lines.get(mCurrentPlayLine - 1).content,
                            progress,
                            width,
                            width * 0.5f,
                            mHighLightTextHeight - mHighLightTextPy + paddingTop);
                }
            }
            if (mCurrentPlayLine < mLyricInfo.song_lines.size()) {
                // 绘制中间行
                canvas.drawText(mLyricInfo.song_lines.get(mCurrentPlayLine).content, width * 0.5f,
                        mHighLightTextHeight + mLineSpace + mDefaultTextHeight - mDefaultTextPy + paddingTop, mDefaultTextPaint);
            }
            if (mCurrentPlayLine + 1 < mLyricInfo.song_lines.size()) {

                // 绘制底下行
                if(mLinesPattern == LinesPattern.THREE_LINES) {
                    canvas.drawText(mLyricInfo.song_lines.get(mCurrentPlayLine + 1).content, width * 0.5f,
                        mHighLightTextHeight + mDefaultTextHeight + mShaderTextHeight - mShaderTextPy + mLineSpace * 2 + paddingTop, mShaderTextPaint);
                }
            }
            if (mSliding && mCurrentPlayLine + 2 < mLyricInfo.song_lines.size()) {
                // 绘制最底下行
                mBottomTextPaint.setAlpha(mBottomTextAlpha);
                if (mLinesPattern == LinesPattern.THREE_LINES) {
                    canvas.drawText(mLyricInfo.song_lines.get(mCurrentPlayLine + 2).content, width * 0.5f,
                        mHighLightTextHeight + mDefaultTextHeight + mShaderTextHeight * 2 - mShaderTextPy + mLineSpace * 3 + paddingTop, mBottomTextPaint);
                } else {
                    canvas.drawText(mLyricInfo.song_lines.get(mCurrentPlayLine + 1).content, width * 0.5f,
                            mHighLightTextHeight + mDefaultTextHeight + mShaderTextHeight * 1 - mShaderTextPy + mLineSpace * 2 + paddingTop, mBottomTextPaint);
                }
            }
        } else {
            // 绘制提示语
            canvas.drawText(mDefaultHint, width * 0.5f, (height - mHintTextHeight) * 0.5f, mHintTextPaint);
        }
    }

    /**
     * 逐行歌词核心高亮方法
     */
    private void drawHighLightLrcRow(Canvas canvas, String text, float rowX, float rowY) {
        canvas.drawText(text, rowX, rowY, mHighLightTextPaint);
    }

    /**
     * 逐字歌词核心高亮方法
     */
    private void drawKaraokeHighLightLrcRow(Canvas canvas, String text, float progress, int width, float rowX, float rowY) {
        // 保存临时变量 等会儿需要还原，默认文本画笔字体大小
        float defaultTextSize = mDefaultTextPaint.getTextSize();
        mDefaultTextPaint.setTextSize(mHighLightTextPaint.getTextSize());

        int highLineWidth = (int) mDefaultTextPaint.measureText(text);
        float location = progress * highLineWidth;

        //如果歌词长于屏幕宽度就需要滚动
        if(highLineWidth > rowX * 2){
            if(location < rowX * 2 / mScale){
                //歌词当前播放位置未到屏幕1/mScale处不需要滚动
                rowX = (float) (highLineWidth / 2.0);
            }else{
                //歌词当前播放位置超过屏幕1/mScale处开始滚动，滚动到歌词结尾到达屏幕边缘时停止滚动。
                float offsetX = location - (rowX * 2 / mScale);
                float widthGap = highLineWidth - rowX * 2;
                if(offsetX < widthGap) {
                    rowX = highLineWidth / 2.0f - offsetX;
                }else{
                    rowX = highLineWidth / 2.0f - widthGap;
                }
            }
        }

        // 先画一层普通颜色的
        canvas.drawText(text, rowX, rowY, mDefaultTextPaint);

        // 再画一层高亮颜色的

        float leftOffset = rowX - highLineWidth / 2f;
        // 高亮的宽度
        int highWidth = (int) (progress * highLineWidth);

        if (highWidth > 1 && ((int) (rowY * 2 ) )> 1) {
            // 用bitmap缓存动画的增量过程
            // 每次绘制都基于当前百分比绘制剪裁后的text文本
            // 动画连起来就是一个逐字歌词的过渡动画了
            Bitmap textBitmap = Bitmap.createBitmap(highWidth, (int) rowY * 2, Bitmap.Config.ARGB_8888);
            Canvas textCanvas = new Canvas(textBitmap);
            textCanvas.drawText(text, highLineWidth / 2f, rowY, mHighLightTextPaint);

            canvas.drawBitmap(textBitmap, leftOffset, 0, mHighLightTextPaint);
        }

        // 还原，默认文本画笔字体大小
        mDefaultTextPaint.setTextSize(defaultTextSize);
    }

    /**
     * 刷新View
     */
    private void invalidateView() {
        if (Looper.getMainLooper() == Looper.myLooper()) {
            //  当前线程是主UI线程，直接刷新。
            invalidate();
        } else {
            //  当前线程是非UI线程，post刷新。
            postInvalidate();
        }
    }

    /**
     * 执行滚动动效
     */
    private void smoothScroll(final int toPosition) {
        mSliding = true;
        // 设置字体颜色
        mHighLightTextPaint.setColor(mHighLightColor);
        mDefaultTextPaint.setColor(mDefaultColor);
        mShaderTextPaint.setColor(mShaderColor);
        mShaderTextPaint.setAlpha(255);

        // 数值计算动效
        valueAnimator = ValueAnimator.ofFloat(0, 1);
        long duration = ZegoLyricViewHelper.getScrollDuration(mLyricInfo, mCurrentPlayLine, toPosition);

        valueAnimator.setDuration(Math.min(duration, MAX_SMOOTH_SCROLL_DURATION));
        valueAnimator.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator animation) {
                // 计算所有动效数值
                // 计算字体大小
                mHighLightTextPaint.setTextSize((mDefaultTextSizePx - mHighLightTextSizePx) * animation.getAnimatedFraction() + mHighLightTextSizePx);
                mDefaultTextPaint.setTextSize((mHighLightTextSizePx - mDefaultTextSizePx) * animation.getAnimatedFraction() + mDefaultTextSizePx);
                mShaderTextPaint.setTextSize((mDefaultTextSizePx - mShaderTextSizePx) * animation.getAnimatedFraction() + mShaderTextSizePx);
                // 计算歌词位移
                mHighLightTextPy = (mHighLightTextHeight + mLineSpace) * animation.getAnimatedFraction();
                mDefaultTextPy = (mDefaultTextHeight + mLineSpace) * animation.getAnimatedFraction();
                mShaderTextPy = (mShaderTextHeight + mLineSpace) * animation.getAnimatedFraction();
                // 计算透明度
                mBottomTextAlpha = (int) (mShaderColorAlpha * animation.getAnimatedFraction());
                invalidateView();
            }
        });
        valueAnimator.addListener(new AnimatorListenerAdapter() {

            @Override
            public void onAnimationEnd(Animator animation) {
                // 重置变量
                if (mCurrentPlayLine != toPosition) {
                    mCurrentPlayLine = toPosition;
                    // 重置字体大小
                    mHighLightTextPaint.setTextSize(mHighLightTextSizePx);
                    mDefaultTextPaint.setTextSize(mDefaultTextSizePx);
                    mShaderTextPaint.setTextSize(mShaderTextSizePx);
                    // 重置偏移量
                    mHighLightTextPy = 0;
                    mDefaultTextPy = 0;
                    mShaderTextPy = 0;
                    // 重置透明度
                    mBottomTextAlpha = 0;
                    // 重新设置字体颜色
                    mHighLightTextPaint.setColor(mHighLightColor);
                    mDefaultTextPaint.setColor(mDefaultColor);
                    mShaderTextPaint.setColor(mShaderColor);
                    mShaderTextPaint.setAlpha(mShaderColorAlpha);
                    // 重置滑动标识
                    mSliding = false;
                    invalidateView();
                }
            }
        });
        valueAnimator.start();
    }

    /**
     * 判断是否可以进行滑动
     */
    private boolean isScrollable() {
        return mLyricInfo != null && mLyricInfo.song_lines != null && mLyricInfo.song_lines.size() > 0;
    }

    /**
     * 根据当前给定的时间戳滑动到指定位置
     *
     * @param time 时间戳
     */
    private void scrollToCurrentTimeMillis(long time) {
        int position = 0;
        if (isScrollable()) {
            for (int i = 0, size = mLineCount; i < size; i++) {
                LineInfo lineInfo = mLyricInfo.song_lines.get(i);
                if (lineInfo != null && lineInfo.start > time) {
                    position = i;
                    break;
                }
                if (i == mLineCount - 1) {
                    position = mLineCount;
                }
            }
        }

        if (mCurrentPlayLine != position && !mSliding) {
            smoothScroll(position);
        }
    }

    /**
     * 换行判断
     */
    private void lineFeed(long time){

        if(mode != MODE_HIGH_LIGHT_KRC){
            return;
        }

        final OnLyricLineFinishedListener lyricLineFeedListener = mOnLyricLineFinishedListener;
        if(lyricLineFeedListener != null) {
            int position = getPosition(time);

            if ( mLineCount > 0) {

                    if (mLineFeedPosition + 1 != position) {
                        mLineFeedPosition = position;
                        return;
                    }


                    if (position == mLineCount - 1) {

                        mLineFeedPosition = position;
                        if(isNotLineFeed(position)) {
                            addLineFeed(position);
                            lyricLineFeedListener.onLyricLineFinished(position,mLyricInfo.song_lines.get(position));
                        }
                    } else {
                        long endTime = 0;
                        if(position >= 0) {
                            LineInfo lineInfo = mLyricInfo.song_lines.get(position);
                            endTime = lineInfo.start + lineInfo.duration;
                        }
                        long lineDurationTime = mLyricInfo.song_lines.get(position + 1).start - endTime;
                        if ((time - (endTime)) >= (TOLERANCE_TIME < lineDurationTime ? TOLERANCE_TIME : lineDurationTime)) {
                            mLineFeedPosition = position;
                            if(isNotLineFeed(position)) {
                                addLineFeed(position);
                                lyricLineFeedListener.onLyricLineFinished(position,mLyricInfo.song_lines.get(position));
                            }
                        }
                    }

                }

        }
    }

    /**
     *
     * @param time 时间
     * @return 最后发生换行的position 从0开始
     */
    private int getPosition(long time){
        int position = -1;
        for (int i = 0, size = mLineCount; i < size; i++) {
            LineInfo lineInfo = mLyricInfo.song_lines.get(i);
            if (lineInfo != null ) {
                long endTime = (lineInfo.start + lineInfo.duration);
                if(lineInfo.words != null && lineInfo.words.size() > 0){
                    WordInfo wordInfo = lineInfo.words.get(lineInfo.words.size() -1);
                    if(wordInfo != null){
                        endTime = lineInfo.start + wordInfo.duration + wordInfo.offset;
                    }
                }
                if(time >= endTime) {
                    position = i;
                }
            }
        }

        return position;
    }

    /**
     * 判断是否是在间隔时间内触发同一行的换行回调
     * @param position 触发回调的行，从0开始
     * @return false 表示是在间隔时间内触发
     */
    private boolean isNotLineFeed(int position){

        boolean isNotLineFeed = true;
        long currentTime = System.currentTimeMillis();
        Iterator<LyricLineFeed> iterator = mLineFeedList.iterator();
        while (iterator.hasNext()) {
            LyricLineFeed lineFeed = iterator.next();
            if(currentTime > lineFeed.getTime() + mLineIntervalTime){
                iterator.remove();
            }else {
                if(lineFeed.position == position){
                    isNotLineFeed = false;
                }
            }

        }

        return isNotLineFeed;
    }

    private void addLineFeed(int position){
        LyricLineFeed lyricLineFeed = new LyricLineFeed();
        lyricLineFeed.setPosition(position);
        lyricLineFeed.setTime(System.currentTimeMillis());
        mLineFeedList.add(lyricLineFeed);
    }



    /**
     * 重置歌词内容
     */
    private void resetLyricInfo() {
        if (mLyricInfo != null) {
            if (mLyricInfo.song_lines != null) {
                mLyricInfo.song_lines.clear();
                mLyricInfo.song_lines = null;
            }
            mLyricInfo = null;
        }
    }

    /**
     * 初始化控件
     */
    private void resetView() {
        resetLyricInfo();
        invalidateView();
        // 停止歌词滚动动效
        if (valueAnimator != null) {
            valueAnimator.cancel();
        }
        mLineCount = 0;
        mDefaultTextPy = 0;
        mHighLightTextPy = 0;
        mShaderTextPy = 0;
    }

    private float getRawSize(int unit, float size) {
        Context context = getContext();
        Resources resources;
        if (context == null) {
            resources = Resources.getSystem();
        } else {
            resources = context.getResources();
        }
        return TypedValue.applyDimension(unit, size, resources.getDisplayMetrics());
    }

    private static class LyricLineFeed{
        private long time;
        private int position;

        public long getTime() {
            return time;
        }

        public void setTime(long time) {
            this.time = time;
        }

        public int getPosition() {
            return position;
        }

        public void setPosition(int position) {
            this.position = position;
        }
    }

    /*
     * ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------ *
     *                                                                                             对外API                                                                                        *
     * ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------ *
     * */

    /**
     * 设置逐行、逐字歌词
     *
     * @param lrc info model
     */
    public void setupLyric(ZGKTVLyric lrc) {
        mCurrentPlayLine = -1;
        if (lrc != null) {
            if (!lrc.isHasKrcFormat()) {
                // 逐行歌词数据源
                mLyricInfo = ZegoLyricViewHelper.parseLrcLyric(lrc);
                mode = MODE_HIGH_LIGHT_NORMAL;
            } else {
                // 逐字歌词数据源
                mLyricInfo = ZegoLyricViewHelper.parseKrcLyric(lrc);
                mode = MODE_HIGH_LIGHT_KRC;
            }
            mLineCount = mLyricInfo.song_lines.size();
        } else {
            mDefaultHint = "暂无歌词";
        }
        clearLyricLineFeed();
        invalidateView();
    }

    /**
     * 设置当前时间显示位置
     *
     * @param current 时间戳
     */
    public void setCurrentTimeMillis(long current) {
        mCurrentTimeMillis = current;
        scrollToCurrentTimeMillis(current);
        lineFeed(current);
        invalidateView();
    }

    /**
     * 重置、设置歌词内容被重置后的提示内容
     *
     * @param message 提示内容
     */
    public void reset(String message) {

        mDefaultHint = message;
        resetView();
        clearLyricLineFeed();
    }

    /**
     * 设置提示语颜色
     */
    public void setHintTextColor(int color) {
        if (mHintColor != color) {
            mHintColor = color;
            mHintTextPaint.setColor(color);
        }
    }

    /**
     * 设置默认文字颜色
     */
    public void setDefaultTextColor(int color) {
        if (mDefaultColor != color) {
            mDefaultColor = color;
            mDefaultTextPaint.setColor(color);
        }
    }

    /**
     * 设置阴影文字颜色
     */
    public void setShaderTextColor(int color, int alpha) {
        if (mShaderColor != color && mShaderColorAlpha != alpha) {
            mShaderColor = color;
            mShaderColorAlpha = alpha;
            mShaderTextPaint.setColor(color);
            mShaderTextPaint.setAlpha(mShaderColorAlpha);
            mBottomTextPaint.setColor(color);
        }
    }

    /**
     * 设置高亮字体颜色
     */
    public void setHighLightTextColor(int color) {
        if (mHighLightColor != color) {
            mHighLightColor = color;
            mHighLightTextPaint.setColor(color);
        }
    }

    /**
     * 设置提示文本内容字体大小
     */
    public void setHintTextSizeSp(float size) {
        float textSizePx = getRawSize(TypedValue.COMPLEX_UNIT_SP, size);
        mHintTextSizePx = textSizePx;
        mHintTextPaint.setTextSize(textSizePx);
        Rect lineBound = new Rect();
        mHintTextPaint.getTextBounds(mDefaultHint, 0, mDefaultHint.length(), lineBound);
        mHintTextHeight = lineBound.height();
        invalidateView();
    }

    /**
     * 设置默认文本内容字体大小
     */
    public void setDefaultTextSizeSp(float size) {
        float textSizePx = getRawSize(TypedValue.COMPLEX_UNIT_SP, size);
        mDefaultTextSizePx = textSizePx;
        mDefaultTextPaint.setTextSize(textSizePx);
        Rect lineBound = new Rect();
        mDefaultTextPaint.getTextBounds(mDefaultHint, 0, mDefaultHint.length(), lineBound);
        mDefaultTextHeight = lineBound.height();
        invalidateView();
    }

    /**
     * 设置阴影字体大小
     */
    public void setShaderTextSizeSp(float size) {
        float textSizePx = getRawSize(TypedValue.COMPLEX_UNIT_SP, size);
        mShaderTextSizePx = textSizePx;
        mShaderTextPaint.setTextSize(textSizePx);
        mBottomTextPaint.setTextSize(textSizePx);
        Rect lineBound = new Rect();
        mShaderTextPaint.getTextBounds(mDefaultHint, 0, mDefaultHint.length(), lineBound);
        mShaderTextHeight = lineBound.height();
        invalidateView();
    }

    /**
     * 设置高亮字体大小
     */
    public void setHighLightTextSizeSp(float size) {
        float textSizePx = getRawSize(TypedValue.COMPLEX_UNIT_SP, size);
        mHighLightTextSizePx = textSizePx;
        mHighLightTextPaint.setTextSize(textSizePx);
        Rect lineBound = new Rect();
        mHighLightTextPaint.getTextBounds(mDefaultHint, 0, mDefaultHint.length(), lineBound);
        mHighLightTextHeight = lineBound.height();
        invalidateView();
    }

    public void setLineSpaceDp(float size) {
        mLineSpace = getRawSize(TypedValue.COMPLEX_UNIT_DIP, size);
        invalidateView();
    }

    /**
     * 设置同行触发回调间隔时间(单位毫秒),两次同一行的换行回调间隔至少n毫秒才会触发，避免播放器同步seek造成的同一行触发多次换行回调
     * 默认: 500
     */
    public void setLineIntervalTime(int intervalTime){
        mLineIntervalTime = intervalTime;
    }

    /**
     * 重置换行记录，保证能够立马触发换行回调，通常在播放器手动seek时调用
     * 外部需要保证在此方法调用到播放器seek结束这段时间内不调用setCurrentTimeMillis()
     */
    public void clearLyricLineFeed(){
        mLineFeedPosition = -1;
        mLineFeedList.clear();
    }


    /**
     * 设置歌词换行回调监听。只在逐字歌词模式才会生效
     */
    public void setOnLyricFinishLineListener(OnLyricLineFinishedListener onLyricLineFinishedListener) {
        this.mOnLyricLineFinishedListener = onLyricLineFinishedListener;
    }

    /**
     * 设置行模式
     * @param linesPattern
     */
    public void setLinesPattern(LinesPattern linesPattern){
        mLinesPattern = linesPattern;
        invalidateView();
    }

    public enum LinesPattern{
        TEO_LINES,
        THREE_LINES
    }

}
