package im.zego.lyricview;

import android.util.Log;

import java.util.ArrayList;
import java.util.List;

import im.zego.lyricview.model.LineInfo;
import im.zego.lyricview.model.LyricInfo;
import im.zego.lyricview.model.WordInfo;
import im.zego.lyricview.model.ZGKTVLyric;

public class ZegoLyricViewHelper {
    private static final String TAG = "ZegoLyricViewHelper";

    /**
     * 逐字播放模式下，根据当前播放的时间戳计算出这一行文字的百分比进度
     * 因为一行中不同的词语占用的时间比重是不均匀的，所以不是一个简单的线性函数（是一个折线函数，单个词语视为线性的），需要单独处理
     *
     * @param currentTimeMillis 当前播放的时间戳
     * @param lineInfo          当前行model
     */
    public static float calculateCurrentKrcProgress(long currentTimeMillis, LineInfo lineInfo) {
        List<WordInfo> words = lineInfo.words;
        long offsetTime = currentTimeMillis - lineInfo.start;

        WordInfo lastWord;
        // 念完所有字所花费的总时长
        long allWordDuration = 0L;
        if (words.size() > 0) {
            lastWord = words.get(words.size() - 1);
            allWordDuration = lastWord.offset + lastWord.duration;
        }

        float progressAll = 0f;
        if (offsetTime < allWordDuration) {
            for (int i = 0; i < words.size(); i++) {
                WordInfo currentWordInfo = words.get(i);
                if (offsetTime >= currentWordInfo.offset && offsetTime <= currentWordInfo.offset + currentWordInfo.duration) {
                    // 在这个词组区间内
                    // 算出之前所有词组的时间占比
                    float progressBefore = i / (float) words.size();
                    // 这个词组占的比重，按份数来算
                    float percent = 1 / (float) words.size();
                    // 在计算当前时间戳在这个词组内的时间占比，线性
                    float progressCurrentWord = (offsetTime - currentWordInfo.offset) / (float) currentWordInfo.duration;
                    // 这两个progress加起来就是总的时间百分比
                    progressAll = progressBefore + progressCurrentWord * percent;
                    break;
                } else if (i < words.size() - 1) {
                    // 不是最后一个字
                    WordInfo nextWordInfo = words.get(i + 1);
                    // 时间在两个字之间的时间间隔
                    // 这里按理来说是不应该被高亮的时间段，即百分比计算出来是不变的
                    if (offsetTime > currentWordInfo.offset + currentWordInfo.duration && offsetTime < nextWordInfo.offset) {
                        progressAll = (i + 1) / (float) words.size();
                    }
                }
            }
        } else {
            progressAll = 1f;
        }
        return progressAll;
    }

    /**
     * 解析逐字歌词数据
     * @param lrc 歌词原始model
     */
    public static LyricInfo parseKrcLyric(ZGKTVLyric lrc) {
        LyricInfo lyricInfo = new LyricInfo();
        lyricInfo.song_lines = new ArrayList<>();
        lyricInfo.song_offset = lrc.getKrcFormatOffset();
        List<ZGKTVLyric.LyricSingleLineModel> singleLineModels = lrc.getLines();
        for (int i = 0; i < singleLineModels.size(); i++) {
            ZGKTVLyric.LyricSingleLineModel singleLineModel = singleLineModels.get(i);

            // 解析每行歌词
            LineInfo lineInfo = new LineInfo();
            lineInfo.start = singleLineModel.getBeginTime();
            lineInfo.start -= lrc.getKrcFormatOffset();
            lineInfo.duration = singleLineModel.getDuration();
            lineInfo.content = singleLineModel.getContent();

            // 解析每行歌词词组
            lineInfo.words = new ArrayList<>();
            List<ZGKTVLyric.LyricWordsModel> words = singleLineModel.getWords();
            for (int j = 0; j < words.size(); j++) {
                ZGKTVLyric.LyricWordsModel wordModel = words.get(j);

                WordInfo wordInfo = new WordInfo();
                wordInfo.offset = wordModel.getOffset();
                wordInfo.duration = wordModel.getDuration();
                wordInfo.word = wordModel.getWord();

                lineInfo.words.add(wordInfo);
            }

            lyricInfo.song_lines.add(lineInfo);
        }

        return lyricInfo;
    }

    /**
     * 解析逐行歌词数据
     * @param lrc 歌词原始model
     */
    public static LyricInfo parseLrcLyric(ZGKTVLyric lrc) {
        LyricInfo lyricInfo = new LyricInfo();
        lyricInfo.song_lines = new ArrayList<>();
        String[] lines = lrc.getLrcFormat().split("\r\n");
        for (String line : lines) {
            analyzeLyric(lyricInfo, line);
        }
        return lyricInfo;
    }

    /**
     * 逐行解析歌词内容
     */
    private static void analyzeLyric(LyricInfo lyricInfo, String line) {
        if (line == null) {
            Log.e(TAG, "analyzeLyric line = null ");
            return;
        }
        int index = line.lastIndexOf("]");

        if (line.startsWith("[offset:")) {
            // 时间偏移量
            String string = line.substring(8, index).trim();
            lyricInfo.song_offset = Long.parseLong(string);
            return;
        }
        if (line.startsWith("[ti:")) {
            // title 标题
            lyricInfo.song_title = line.substring(4, index).trim();
            return;
        }
        if (line.startsWith("[ar:")) {
            // artist 作者
            lyricInfo.song_artist = line.substring(4, index).trim();
            return;
        }
        if (line.startsWith("[al:")) {
            // album 所属专辑
            lyricInfo.song_album = line.substring(4, index).trim();
            return;
        }
        if (line.startsWith("[by:")) {
            return;
        }
        if (index >= 9 && line.trim().length() > index + 1) {
            // 歌词内容
            LineInfo lineInfo = new LineInfo();
            lineInfo.content = line.substring(index + 1);
            lineInfo.start = measureStartTimeMillis(line.substring(line.indexOf("["), index));
            lyricInfo.song_lines.add(lineInfo);
        }
    }

    /**
     * 从字符串中获得时间值
     */
    private static long measureStartTimeMillis(String str) {
        // 分
        long minute = Long.parseLong(str.substring(1, 3));
        // 秒
        long second = Long.parseLong(str.substring(4, 6));
        // 毫秒
        long millisecond = Long.parseLong(str.substring(7, 9));
        //
        return millisecond + second * 1000 + minute * 60 * 1000;
    }

    public static long getScrollDuration(LyricInfo lyricInfo, int fromPosition, int toPosition) {
        long duration = 0;
        try {
            LineInfo fromLineInfo = lyricInfo.song_lines.get(fromPosition);
            LineInfo toLineInfo = lyricInfo.song_lines.get(toPosition);
            duration = toLineInfo.start - (fromLineInfo.start + fromLineInfo.duration);
        } catch (Exception ignored) {
        }
        return duration > 0 ? duration : 0;
    }
}
